! $Id: parameters.SM.omega.f90,v 1.4 2006/06/16 13:31:48 kilian Exp $
!
! Copyright (C) 2000 by Thorsten Ohl <ohl@hep.tu-darmstadt.de>
! and others
!
! O'Mega is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by 
! the Free Software Foundation; either version 2, or (at your option)
! any later version.
!
! O'Mega is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  

module parameters_test
  use kinds !NODEP!
  use constants !NODEP!

  implicit none
  private

  real(default), dimension(27), public :: mass, width
  real(default), public :: e, g, e_em
  complex(default), public :: gs, igs
  real(default), public :: sinthw, costhw, sin2thw, tanthw
  real(default), public :: qelep, qeup, qedwn
  real(default), public :: ttop, tbot, tch, ttau, tw
  real(default), public :: ltop, lbot, lc, ltau, lw
  complex(default), public :: qlep, qup, qdwn, gcc, qw, &
       gzww, gwww, ghww, ghhww, ghzz, ghhzz, &
       ghbb, ghtt, ghcc, ghtautau, gh3, gh4, &
       ghgaga, ghgaz, ghgg, ghmm, & 		
       iqw, igzww, igwww, gw4, gzzww, gazww, gaaww
  real(default), public :: vev
  complex(default), dimension(2), public :: &
       gncneu, gnclep, gncup, gncdwn
  real(default), public :: xi0 = 0, xipm = 0

  integer, parameter, public :: &
       n0 = 5, nloop = 2 
  real(default), parameter :: &
       acc = 1.e-12_default
  real(default), parameter :: &
       asmz = 0.118_default

  public :: import_from_whizard

contains

! *********************************************************************
! auxiliary functions for anomaly Higgs couplings
  function faux (x) result (y)
    real(default) :: x
    complex(default) :: y
    if (1 <= x) then
       y = asin(sqrt(1/x))**2
    else
       y = - 1/4.0_default * (log((1 + sqrt(1 - x))/ &
            (1 - sqrt(1 - x))) - cmplx (0.0_default, PI, kind=default))**2
    end if
  end function faux

  function fonehalf (x) result (y)
    real(default), intent(in) :: x
    complex(default) :: y
    if (x==0) then
       y = 0
    else
       y = - 2.0_default * x * (1 + (1 - x) * faux(x))
    end if
  end function fonehalf

  function fone (x) result  (y)
    real(default), intent(in) :: x
    complex(default) :: y
    if (x==0) then
       y = 2.0_default
    else
       y = 2.0_default + 3.0_default * x + &
            3.0_default * x * (2.0_default - x) * &
            faux(x)
    end if
  end function fone

  function gaux (x) result (y)
    real(default), intent(in) :: x
    complex(default) :: y
    if (1 <= x) then
       y = sqrt(x - 1) * asin(sqrt(1/x))
    else
       y = sqrt(1 - x) * (log((1 + sqrt(1 - x)) / &
            (1 - sqrt(1 - x))) - cmplx (0.0_default, PI, kind=default)) / 2
    end if
  end function gaux

  function i1 (a,b) result (y)
    real(default), intent(in) :: a,b
    complex(default) :: y
    y = a*b/2.0_default/(a-b) + a**2 * b**2/2.0_default/(a-b)**2 * &
         (faux(a) - faux(b)) + &
         a**2 * b/(a-b)**2 * (gaux(a) - gaux(b))
  end function i1

  function i2 (a,b) result (y) 
    real(default), intent(in) :: a,b
    complex(default) :: y
    y = - a * b / 2.0_default / (a-b) * (faux(a) - faux(b)) 
  end function i2

  function b0 (nf) result (bnull)
    integer, intent(in) :: nf
    real(default) :: bnull
    bnull = 33.0_default - 2.0_default * nf
  end function b0

  function b1 (nf) result (bone)
    integer, intent(in) :: nf
    real(default) :: bone
    bone = 6.0_default * (153.0_default - 19.0_default * nf)/b0(nf)**2
  end function b1

  function aa (nf) result (aaa)
    integer, intent(in) :: nf
    real(default) :: aaa
    aaa = 12.0_default * PI / b0(nf)
  end function aa

  function bb (nf) result (bbb)
    integer, intent(in) :: nf
    real(default) :: bbb
    bbb = b1(nf) / aa(nf)
  end function bb

! *********************************************************************

! Parameters are transferred as a simple real array
  subroutine import_from_whizard (par_array)
    real(default), dimension(17), intent(in) :: par_array
     !!! This corresponds to 1/alpha = 137.03598949333
    real(default), parameter :: &
         alpha = 1.0_default/137.03598949333_default
    real(default), parameter :: &
         asmz = 0.118_default
    
! Temporary for intermediate storage, no longer used by WHIZARD
    type :: parameter_set
       real(default) :: ms = 0
       real(default) :: mc = 0
       real(default) :: mb = 0
       real(default) :: mtop = 175
       real(default) :: wtop = 0
       real(default) :: me = 0
       real(default) :: mmu = 0
       real(default) :: mtau = 0
       real(default) :: mz = 91
       real(default) :: wz = 0
       real(default) :: mw = 80
       real(default) :: ww = 0
       real(default) :: mh = 120
       real(default) :: wh = 0
       real(default) :: alpha_s = 0
       real(default) :: gg = 0
       real(default) :: ee = 0.3
       real(default) :: sw = 0.2
       real(default) :: cw = 1 - 0.2**2
       real(default) :: khgaga = 0
       real(default) :: khgaz = 0
       real(default) :: khgg = 0
    end type parameter_set

    type(parameter_set) :: par

! Assign relevant parameters to temporary parameter set
    par%alpha_s = par_array(1)
    par%ms = par_array(2)
    par%mc = par_array(3)
    par%mb = par_array(4)
    par%mtop = par_array(5)
    par%wtop = par_array(6)
    par%mz = par_array(7)
    par%wz = par_array(8)
    par%mw = par_array(9)
    par%ww = par_array(10)
    par%mh = par_array(11)
    par%wh = par_array(12)
    ! par%gf = par_array(13)
    par%gg = par_array(14)
    par%cw = par_array(15)
    par%sw = par_array(16)
    par%ee = par_array(17)

! The rest is the original version
    e_em = sqrt(4.0_default * PI * alpha)
    mass(1:27) = 0
    width(1:27) = 0
    mass(3) = par%ms
    mass(4) = par%mc
    mass(5) = par%mb
    mass(6) = par%mtop
    width(6) = par%wtop
    mass(11) = par%me
    mass(13) = par%mmu
    mass(15) = par%mtau
    mass(23) = par%mZ
    width(23) = par%wZ
    mass(24) = par%mW
    width(24) = par%wW
    mass(25) = par%mH
    width(25) = par%wH
    mass(26) =  xi0 * mass(23)
    width(26) =  0
    mass(27) =  xipm * mass(24)
    width(27) =  0
    ttop = 4.0_default * mass(6)**2 / mass(25)**2
    tbot = 4.0_default * mass(5)**2 / mass(25)**2
    tch  = 4.0_default * mass(4)**2 / mass(25)**2
    ttau = 4.0_default * mass(15)**2 / mass(25)**2
    tw   = 4.0_default * mass(24)**2 / mass(25)**2  
    ltop = 4.0_default * mass(6)**2 / mass(23)**2
    lbot = 4.0_default * mass(5)**2 / mass(23)**2  
    lc   = 4.0_default * mass(4)**2 / mass(23)**2
    ltau = 4.0_default * mass(15)**2 / mass(23)**2
    lw   = 4.0_default * mass(24)**2 / mass(23)**2
    vev = 2 * mass(24) * par%sw / par%ee 
    e = par%ee
    sinthw = par%sw
    sin2thw = sinthw**2
    costhw = par%cw
    tanthw = sinthw/costhw
    qelep = - 1
    qeup = 2.0_default / 3.0_default
    qedwn = - 1.0_default / 3.0_default
    g = e / sinthw
    gcc = - g / 2 / sqrt (2.0_default)
    gncneu(1) = - g / 2 / costhw * ( + 0.5_default)
    gnclep(1) = - g / 2 / costhw * ( - 0.5_default - 2 * qelep * sin2thw)
    gncup(1)  = - g / 2 / costhw * ( + 0.5_default - 2 * qeup  * sin2thw)
    gncdwn(1) = - g / 2 / costhw * ( - 0.5_default - 2 * qedwn * sin2thw)
    gncneu(2) = - g / 2 / costhw * ( + 0.5_default)
    gnclep(2) = - g / 2 / costhw * ( - 0.5_default)
    gncup(2)  = - g / 2 / costhw * ( + 0.5_default)
    gncdwn(2) = - g / 2 / costhw * ( - 0.5_default)
    qlep = - e * qelep
    qup = - e * qeup
    qdwn = - e * qedwn
    qw = e
    iqw = (0,1)*qw
    gzww = g * costhw
    igzww = (0,1)*gzww
    gwww = g
    igwww = (0,1)*gwww
    gw4 = gwww**2
    gzzww = gzww**2
    gazww = gzww * qw
    gaaww = qw**2
    ghww = mass(24) * g
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!! This is for the old SM3:
    !!! ghhww = (0,1) * g / Sqrt(2.0_default)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ghhww = g**2 / 2.0_default
    ghzz = mass(23) * g / costhw
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!! This is for the old SM3:
    !!! ghhzz = (0,1) * g / costhw / Sqrt(2.0_default)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ghhzz = g**2 / 2.0_default / costhw**2
    ghtt = - mass(6) / vev
    ghbb = - mass(5) / vev
    ghcc = - mass(4) / vev
    ghtautau = - mass(15) / vev
    ghmm = - mass(13) / vev
    gh3 = - 3 * mass(25)**2 / vev
    !!! gh4 = mass(25) / vev !!! Old SM3
    gh4 = - 3 * mass(25)**2 / vev**2
    gs = par%gg / sqrt(2.0_default)
    igs = (0.0_default, 1.0_default) * gs    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!! Higgs anomaly couplings
    !!! SM LO loop factor (top,bottom,W)
    ghgaga = alpha / vev / 2.0_default / PI * &
         Abs(( 4.0_default * (fonehalf(ttop) + fonehalf(tch)) &
         + fonehalf(tbot)) / 3.0_default + fonehalf(ttau) + fone(tw)) &
         * sqrt(par%khgaga)
    !!! asymptotic limit:
    !!! ghgaga = (par%ee)**2 / vev / &
    !!!      9.0_default / pi**2
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!! SM LO loop factor (only top and W)
    ghgaz = e * e_em / 8.0_default / PI**2 / vev * Abs( &
          ( - 2.0_default + &
          16.0_default/3.0_default * sin2thw) * &
          (i1(ttop,ltop) - i2(ttop,ltop)) / costhw & 
          + ( - 1.0_default + &
          4.0_default/3.0_default * sin2thw) & 
          * (i1(tbot,lbot) - i2(tbot,lbot)) / costhw &
           - costhw * ( 4.0_default * (3.0_default - tanthw**2) * &
           i2(tw,lw) + ((1 + 2.0_default/tw) * tanthw**2 - ( &
           5.0_default + 2.0_default/tw)) * i1(tw,lw)) &
          )/sinthw * sqrt(par%khgaz)
    !!! SM LO order loop factor with 
    !!! N(N)LO K factor = 2.1 (only top)
    !!! Limit of infinite top quark mass:
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!! We use par%gg because of sqrt(2) above
    ghgg = par%gg**2 / vev / 16.0_default / PI**2 * &
         Abs(fonehalf(ttop) + fonehalf(tbot) + fonehalf(tch)) * &
         sqrt(par%khgg)
    !!! ghgg   = (par%gg)**2 / 12.0_default &
    !!!      / vev / pi**2 * 2.1_default
  end subroutine import_from_whizard
end module parameters_test
