! WHIZARD 2.0.0 Mon Aug 24 2009
! 
! (C) 1999-2009 by 
!     Wolfgang Kilian <kilian@hep.physik.uni-siegen.de>
!     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
!     Juergen Reuter <juergen.reuter@physik.uni-freiburg.de>
!     with contributions by Sebastian Schmidt
!
! WHIZARD is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by 
! the Free Software Foundation; either version 2, or (at your option)
! any later version.
!
! WHIZARD is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! This file has been stripped of most comments.  For documentation, refer
! to the source 'whizard.nw'

program main

  use iso_varying_string, string_t => varying_string !NODEP!
  use system_dependencies !NODEP!
  use limits, only: CMDLINE_ARG_LEN
  use diagnostics
  use whizard

  implicit none

! For test code
!   use os_interface
!   use sorting
!   use hashes
!   use parser
!   use expressions
!   use models
!   use colors
!   use cascades
!   use state_matrices
!   use interactions
!   use evaluators
!   use polarizations
!   use hepmc_interface
!   use particles
!   use beams
!   use strfun
!   use phs_forests
!   use process_libraries
!   use hard_interactions
!   use processes
!   use events
!   use commands

!   external :: usage

  ! Main program variable declarations
  character(CMDLINE_ARG_LEN) :: arg
  character(2) :: option
  integer :: i, j, arg_len, arg_status
  logical :: look_for_options
  logical :: interactive
  type(string_t) :: files, this, model, libname, library, libraries, logfile
  type(string_t) :: check, checks
  logical :: rebuild, recompile

  ! Initial values
  look_for_options = .true.
  interactive = .false.
  files = ""
  model = "SM"
  libname = "processes"
  library = ""
  logfile = "whizard.log"
  libraries = ""
  check = ""
  checks = ""
  rebuild = .false.
  recompile = .false.

  ! Read and process options
  i = 0
  SCAN_CMDLINE: do
     i = i + 1
     call get_command_argument (i, arg, arg_len, arg_status)
     select case (arg_status)
     case (0)
     case (-1)
        call msg_error (" Command argument truncated: '" // arg // "'")
     case default
        exit SCAN_CMDLINE
     end select
     if (look_for_options) then
        select case (arg(1:2))
        case ("--")
           select case (trim (arg))
           case ("--version")
              call print_version (); stop
           case ("--help")
              call print_usage (); stop
           case ("--check")
              check = get_option_value (i, trim (arg))
              checks = checks // " " // check
              cycle SCAN_CMDLINE
           case ("--interactive")
              interactive = .true.
              cycle SCAN_CMDLINE
           case ("--library")
              library = get_option_value (i, trim (arg))
              libraries = libraries // " " // library
              cycle SCAN_CMDLINE
           case ("--logfile")
              logfile = get_option_value (i, trim (arg))
              cycle SCAN_CMDLINE
           case ("--no-logfile")
              logfile = ""
              cycle SCAN_CMDLINE
           case ("--model")
              model = get_option_value (i, trim (arg))
              cycle SCAN_CMDLINE
           case ("--rebuild")
              rebuild = .true.
              cycle SCAN_CMDLINE
           case ("--recompile")
              recompile = .true.
              cycle SCAN_CMDLINE
           case default
              call print_usage ()
              call msg_fatal ("Option '" // trim (arg) // "' not recognized")
           end select
        end select
        select case (arg(1:1))
        case ("-")
           j = 1
           if (len_trim (arg) == 1) then
              look_for_options = .false.
           else
              SCAN_SHORT_OPTIONS: do
                 j = j + 1
                 if (j > len_trim (arg)) exit SCAN_SHORT_OPTIONS
                 option = "-" // arg(j:j)
                 select case (option)
                 case ("-V")
                    call print_version (); stop
                 case ("-?", "-h")
                    call print_usage (); stop
                 case ("-i")
                    interactive = .true.
                    cycle SCAN_SHORT_OPTIONS
                 case ("-l")
                    if (j == len_trim (arg)) then
                       library = get_option_value (i, option)
                    else
                       library = trim (arg(j+1:))
                    end if
                    libraries = libraries // " " // library
                    cycle SCAN_CMDLINE
                 case ("-L")
                    if (j == len_trim (arg)) then
                       logfile = get_option_value (i, option)
                    else
                       logfile = trim (arg(j+1:))
                    end if
                    cycle SCAN_CMDLINE
                 case ("-m")
                    if (j < len_trim (arg))  call msg_fatal &
                         ("Option '" // option // "' needs a value")
                    model = get_option_value (i, option)
                    cycle SCAN_CMDLINE
                 case ("-r")
                    rebuild = .true.
                    cycle SCAN_SHORT_OPTIONS
                 case default
                    call print_usage ()
                    call msg_fatal &
                         ("Option '" // option // "' not recognized")
                 end select
              end do SCAN_SHORT_OPTIONS
           end if
        case default
           files = files // " " // trim (arg)
        end select
     else
        files = files // " " // trim (arg)
     end if
  end do SCAN_CMDLINE

  ! Overall initialization
  if (logfile /= "")  call logfile_init (logfile)
  call msg_banner ()
  call whizard_init &
       (preload_model=model, preload_libs=libraries, default_lib=libname, &
        rebuild=rebuild, recompile=recompile)

  ! Run any self-checks (and no commands)
  if (checks /= "") then

     checks = trim (adjustl (checks))
     RUN_CHECKS: do while (checks /= "")
        call split (checks, check, " ")
        call whizard_check (check)
     end do RUN_CHECKS

  ! Process commands from standard input
  else if (.not. interactive .and. files == "") then
     call whizard_process_stdin ()

  ! Process commands from file
  else
     files = trim (adjustl (files))
     SCAN_FILES: do while (files /= "")
        call split (files, this, " ")
        call whizard_process_file (this)
     end do SCAN_FILES

  end if 

  ! Enter an interactive shell if requested
  if (interactive) then
     call whizard_shell ()
  end if

  ! Overall finalization
  call whizard_final ()
  call msg_terminate ()

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
contains

  function get_option_value (i, option) result (string)
    type(string_t) :: string
    integer, intent(inout) :: i
    character(*), intent(in) :: option
    character(CMDLINE_ARG_LEN) :: arg
    character(CMDLINE_ARG_LEN) :: arg_value
    integer :: arg_len, arg_status
    i = i + 1
    call get_command_argument (i, arg_value, arg_len, arg_status)
    select case (arg_status)
    case (0)
    case (-1)
       call msg_error (" Option value truncated: '" // arg // "'")
    case default
       call print_usage ()
       call msg_fatal (" Option '" // trim (option) // "' needs a value")
    end select
    select case (arg(1:1))
    case ("-")
       call print_usage ()
       call msg_fatal (" Option '" // trim (option) // "' needs a value")
    end select
    string = trim (arg_value)
  end function get_option_value

  subroutine print_version ()
    print "(A)", "WHIZARD " // WHIZARD_VERSION 
    print "(A)", "Copyright (C) 1999-2009 Wolfgang Kilian, Thorsten Ohl, Juergen Reuter"
    print "(A)", "This is free software; see the source for copying conditions.  There is NO"
    print "(A)", "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
    print *
  end subroutine print_version

  subroutine print_usage ()
    print "(A)", "WHIZARD " // WHIZARD_VERSION
    print "(A)", "Usage: whizard [OPTIONS] [FILE]"
    print "(A)", "Run WHIZARD with the command list taken from FILE(s)"
    print "(A)", "Options:"
    print "(A)", "-h, --help            display this help and exit"
    print "(A)", "-i, --interactive     run interactively after reading FILE(s)"
    print "(A)", "-l, --library         preload process library NAME"
    print "(A)", "-L, --logfile FILE    write log to FILE (default: 'whizard.log'"
    print "(A)", "-m, --model NAME      preload model NAME (default: 'SM')"
    print "(A)", "    --no-logfile      do not write a logfile"
    print "(A)", "-r, --rebuild         rebuild process code, ignoring any existing library"
    print "(A)", "    --recompile       recompile process code, ignoring any existing library"
    print "(A)", "-V, --version         output version information and exit"
    print "(A)", "-                     further options are taken as filenames"
    print *
    print "(A)", "With no FILE, read standard input."
  end subroutine print_usage

end program main
